@@ -3,8 +3,9 @@
 lazyarray is a Python package that provides a lazily-evaluated numerical array
 class, ``larray``, based on and compatible with NumPy arrays.
 
-Copyright Andrew P. Davison and Joël Chavas 2012-2014
+Copyright Andrew P. Davison, Joël Chavas and Elodie Legouée (CNRS), 2012-2019
 """
+
 from __future__ import division
 import numpy
 import operator
@@ -13,7 +14,15 @@ import collections
 from functools import wraps
 import logging
 
-__version__ = "0.2.7"
+try:
+    from scipy import sparse
+    from scipy.sparse import bsr_matrix, coo_matrix, csc_matrix, csr_matrix, dia_matrix, dok_matrix, lil_matrix
+    have_scipy = True
+except ImportError:
+    have_scipy = False
+
+
+__version__ = "0.3.3"
 
 # stuff for Python 3 compatibility
 try:
@@ -42,7 +51,7 @@ def check_shape(meth):
     """
     @wraps(meth)
     def wrapped_meth(self, val):
-        if isinstance(val, (larray, numpy.ndarray)):
+        if isinstance(val, (larray, numpy.ndarray)) and val.shape:
             if val.shape != self._shape:
                 raise ValueError("shape mismatch: objects cannot be broadcast to a single shape")
         return meth(self, val)
@@ -78,7 +87,8 @@ def partial_shape(addr, full_shape):
         if isinstance(x, (int, long, numpy.integer)):
             return None
         elif isinstance(x, slice):
-            return 1 + ((x.stop or max) - (x.start or 0) - 1) // (x.step or 1)
+            y = min(max, x.stop or max)  # slice limits can go past the bounds
+            return 1 + (y - (x.start or 0) - 1) // (x.step or 1)
         elif isinstance(x, collections.Sized):
             if hasattr(x, 'dtype') and x.dtype == bool:
                 return x.sum()
@@ -135,6 +145,7 @@ def lazy_unary_operation(name):
     return op
 
 
+
 class larray(object):
     """
     Optimises storage of and operations on arrays in various ways:
@@ -150,6 +161,7 @@ class larray(object):
         on different nodes or in different threads.
     """
 
+
     def __init__(self, value, shape=None, dtype=None):
         """
         Create a new lazy array.
@@ -162,6 +174,7 @@ class larray(object):
         and a 1D array when either `i` or `j` or both is a NumPy array (in the
         latter case the two arrays must have equal lengths).
         """
+
         self.dtype = dtype
         self.operations = []
         if isinstance(value, basestring):
@@ -173,19 +186,23 @@ class larray(object):
             self.base_value = value.base_value
             self.dtype = dtype or value.dtype
             self.operations = value.operations  # should deepcopy?
+
         elif isinstance(value, collections.Sized):  # False for numbers, generators, functions, iterators
-            if not isinstance(value, numpy.ndarray):
+            if have_scipy and sparse.issparse(value):  # For sparse matrices
+                self.dtype = dtype or value.dtype
+            elif not isinstance(value, numpy.ndarray):
                 value = numpy.array(value, dtype=dtype)
             elif dtype is not None:
-                assert value.dtype == dtype  # or could convert value to the provided dtype
+               assert numpy.can_cast(value.dtype, dtype, casting='safe')  # or could convert value to the provided dtype
             if shape and value.shape != shape:
                 raise ValueError("Array has shape %s, value has shape %s" % (shape, value.shape))
             self._shape = value.shape
             self.base_value = value
+
         else:
             assert numpy.isreal(value)  # also True for callables, generators, iterators
             self._shape = shape
-            if dtype is None:
+            if dtype is None or isinstance(value, dtype):
                 self.base_value = value
             else:
                 try:
@@ -222,7 +239,9 @@ class larray(object):
                                                                              self.operations)
 
     def _set_shape(self, value):
-        if hasattr(self.base_value, "shape") and self.base_value.shape != value:
+        if (hasattr(self.base_value, "shape") and
+                self.base_value.shape and   # values of type numpy.float have an empty shape
+                    self.base_value.shape != value):
             raise ValueError("Lazy array has fixed shape %s, cannot be changed to %s" % (self.base_value.shape, value))
         self._shape = value
         for op in self.operations:
@@ -255,7 +274,9 @@ class larray(object):
     @property
     def is_homogeneous(self):
         """True if all the elements of the array are the same."""
-        hom_base = isinstance(self.base_value, (int, long, numpy.integer, float, bool)) or type(self.base_value) == self.dtype
+        hom_base = isinstance(self.base_value, (int, long, numpy.integer, float, bool)) \
+                   or type(self.base_value) == self.dtype \
+                   or (isinstance(self.dtype, type) and isinstance(self.base_value, self.dtype))
         hom_ops = all(obj.is_homogeneous for f, obj in self.operations if isinstance(obj, larray))
         return hom_base and hom_ops
 
@@ -336,11 +357,15 @@ class larray(object):
             base_val = self._homogeneous_array(addr) * self.base_value
         elif isinstance(self.base_value, numpy.ndarray):
             base_val = self.base_value[addr]
+        elif have_scipy and sparse.issparse(self.base_value):  # For sparse matrices larr[2, :]
+            base_val = self.base_value[addr]
         elif callable(self.base_value):
             indices = self._array_indices(addr)
             base_val = self.base_value(*indices)
             if isinstance(base_val, numpy.ndarray) and base_val.shape == (1,):
                 base_val = base_val[0]
+        elif hasattr(self.base_value, "lazily_evaluate"):
+            base_val = self.base_value.lazily_evaluate(addr, shape=self._shape)
         elif isinstance(self.base_value, VectorizedIterable):
             partial_shape = self._partial_shape(addr)
             if partial_shape:
@@ -371,7 +396,7 @@ class larray(object):
                 lower = upper = x
             elif isinstance(x, slice):
                 lower = x.start or 0
-                upper = x.stop or size - 1
+                upper = min(x.stop or size - 1, size - 1)  # slices are allowed to go past the bounds
             elif isinstance(x, collections.Sized):
                 if is_boolean_array(x):
                     lower = 0
@@ -425,12 +450,13 @@ class larray(object):
                     x = f(x, arg.evaluate(simplify=simplify))
                 else:
                     x = f(x, arg._partially_evaluate(addr, simplify=simplify))
+
             else:
                 x = f(x, arg)
         return x
 
     @requires_shape
-    def evaluate(self, simplify=False):
+    def evaluate(self, simplify=False, empty_val=0):
         """
         Return the lazy array as a real NumPy array.
 
@@ -443,16 +469,24 @@ class larray(object):
                 x = self.base_value
             else:
                 x = self.base_value * numpy.ones(self._shape, dtype=self.dtype)
-        elif isinstance(self.base_value, (int, long, numpy.integer, float, bool)):
+        elif isinstance(self.base_value, (int, long, numpy.integer, float, bool, numpy.bool_)):
             x = self.base_value * numpy.ones(self._shape, dtype=self.dtype)
         elif isinstance(self.base_value, numpy.ndarray):
             x = self.base_value
         elif callable(self.base_value):
             x = numpy.array(numpy.fromfunction(self.base_value, shape=self._shape, dtype=int), dtype=self.dtype)
+        elif hasattr(self.base_value, "lazily_evaluate"):
+            x = self.base_value.lazily_evaluate(shape=self._shape)
         elif isinstance(self.base_value, VectorizedIterable):
             x = self.base_value.next(self.size)
             if x.shape != self._shape:
                 x = x.reshape(self._shape)
+        elif have_scipy and sparse.issparse(self.base_value):  # For sparse matrices
+            if empty_val!=0:
+                x = self.base_value.toarray((sparse.csc_matrix))
+                x = numpy.where(x, x, numpy.nan)
+            else:
+                x = self.base_value.toarray((sparse.csc_matrix))
         elif isinstance(self.base_value, collections.Iterator):
             x = numpy.fromiter(self.base_value, dtype=self.dtype or float, count=self.size)
             if x.shape != self._shape:
@@ -524,9 +558,10 @@ class VectorizedIterable(object):
     pass
 
 
-# build lazy-array comptible versions of NumPy ufuncs
+# build lazy-array compatible versions of NumPy ufuncs
 namespace = globals()
 for name in dir(numpy):
     obj = getattr(numpy, name)
     if isinstance(obj, numpy.ufunc):
         namespace[name] = _build_ufunc(obj)
+
